from setuptools import setup, find_packages

setup(
    name="marge-mri-example",
    version="0.0.5",
    packages=find_packages(),
    install_requires=[],
    author="Nihal Ouahabi",
    description="Une description de ton package MaRGE",
)

